#!/usr/bin/python3

import unittest, inspect
from avlib import ascanner, tobytes, mail, debug
from avir import *
from aspam import *
from interscan import *

# for testing purposes
debug.set_level(1)
mail.addr = [b"127.0.0.1", 25, b"noname"]

class ScannerTest(unittest.TestCase):
  buffer = b"MZ_executable\0_SomethingFound"
  args = {}

  def assertClean(self, scanner):
      self.assertEqual(
        scanner.scanbuffer(self.buffer, self.args)[0:2], (0.0, b"")
      )

  def assertInf(self, scanner, virus=""):
      self.assertEqual(
        scanner.scanbuffer(self.buffer, self.args)[0:2],
        (1.0, tobytes(virus))
      )

class BasicVirScanners(ScannerTest):
  def test_regexp_scan(self):
      self.assertInf(regexp_scan([["VIR_NAME", self.buffer]]), "VIR_NAME")
      self.assertClean(regexp_scan([["VIR_NAME", "NOT FOUND"]]))
  def test_string_scan(self):
      self.assertInf(string_scan([["VIR_NAME", self.buffer]]), "VIR_NAME")
      self.assertClean(string_scan([["VIR_NAME", "NOT FOUND"]]))
  def test_smtp_comm(self):
      pass # TODO
  def test_const(self):
      self.assertClean(const(0.0))
      self.assertInf(const(1.0, "VIR_NAME"), "VIR_NAME")
  def test_max_file_size(self):
      self.assertInf(max_file_size(len(self.buffer)-1), "FileSizeOverrun")
      self.assertClean(max_file_size(len(self.buffer)+1))
  def test_file_type(self):
      self.assertInf(file_type({"exe": "mexec"}), "mexec")
      self.assertClean(file_type({"doc": "document"}))
  def test_file_magic(self):
      self.assertInf(file_magic({"exe": "application/x-dosexec"}), "exe")
      self.assertClean(file_magic({"doc": "document"}))
  def test_sender_regexp(self):
      self.assertInf(
        sender_regexp([["VIR_NAME", "127.0.0.1"]]), "VIR_NAME"
      )
      self.assertClean(
        sender_regexp([["VIR_NAME", "127.0.0.2"]])
      )

class ClamavScanner(ScannerTest):
  eicar = b"X5O!P%@ AP[4\\PZX54(P^) 7CC)7}$EIC AR-STAN DARD-ANTI VIRUS-TEST-FILE!$H+H*".replace(b" ", b"")

  def setUp(self):
      self.scanner = b2f(libclam())
      self.scanner.reinit()

  def test_libclam(self):
      self.assertClean(self.scanner)
      self.buffer = self.eicar
      self.assertInf(self.scanner, "Eicar-Test-Signature")

if __name__ == "__main__":
  unittest.main()
